// dllmain.cpp : Defines the entry point for the DLL application.
#include "pch.h"
#include "dllmain.h"
#include <string>
#include <vector>

/* Definition and declaration of the OSC Listeners. 
* Contains OSC addresses to listen to and OS message ID to send for each 
* address */
struct Listener {
    std::string address;
    unsigned int message;
};
std::vector<Listener> listeners;
/* Handle to the calling window */
HWND hwnd = NULL;

BOOL APIENTRY DllMain( HMODULE hModule,
                       DWORD  ul_reason_for_call,
                       LPVOID lpReserved
                     )
{
    switch (ul_reason_for_call)
    {
    case DLL_PROCESS_ATTACH:
    case DLL_THREAD_ATTACH:
        break;
    case DLL_THREAD_DETACH:
    case DLL_PROCESS_DETACH:
        close();
        break;
    }
    return TRUE;
}

DLLEXPORT int open(HWND targetWindowHandle, unsigned int port)
{
    hwnd = targetWindowHandle;
    return 0;
}

DLLEXPORT int close()
{
    hwnd = NULL;
    listeners.clear();
    return 0;
}

DLLEXPORT int addListener(std::string address, unsigned int messageID)
{
    listeners.push_back(Listener{ address, messageID });
    return 0;
}

DLLEXPORT int removeListener(std::string address)
{
    UINT result = 1;

    //All OSC addresses have to start with a '/'
    if (address[0] != '/') address.insert(0, "/");

    for (UINT i = 0; i < listeners.size(); i++)
    {
        if (listeners[i].address == address)
        {
            listeners.erase(listeners.begin() + i);
            i--; //one step backward, what previously was i+1 is i after erase().
            result = 0; //Found at least one occurence
        }
    }
    return result;
}

/* Debugging function to test messaging. Probably will be removed later. */
DLLEXPORT int testMsg(HWND windowHandle, unsigned int messageID)
{
    if (PostMessage(windowHandle, messageID, 42, 43))
        return 0;
    else
        return 1;
}
